
import mock
from django.test import TestCase
from django import template
# from django.template import Template, Context
from django.utils import unittest
try:
    from django.test.utils import override_settings
except ImportError:
    def override_settings(*args, **kwargs):
        return unittest.skip("overriding settings is not supported by this django version")

from easy_maps.models import Address

class AddressTests(TestCase):
    fake_default_center  = (1, 2)

    def test_empty_dont_save_on_db(self):
        """If we pass an empty address we don't save nothing in the database"""
        n_addresses_before = len(Address.objects.all())

        simple_template_string = """{% load easy_maps_tags %}
        {% easy_map "" 500 500 10 %}
        """
        t = template.Template(simple_template_string)
        t.render(template.Context({}))

        n_addresses_after = len(Address.objects.all())

        self.assertEqual(n_addresses_after, n_addresses_before)

    @override_settings(EASY_MAPS_CENTER=fake_default_center)
    def test_empty_address_use_defaults(self):
        """When an empty address is passed uses the EASY_MAPS_CENTER setting"""
        a = ""
        simple_template_string = """{%% load easy_maps_tags %%}
        {%% easy_map "%s" 500 500 10 %%}
        """ % a

        address = [None]  # nonlocal

        # below we patch the render_to_string in order to retrieve the map
        # context variable and check its coordinate
        def get_map_context_instance(*args, **kwargs):
            address[0] = kwargs['context_instance']['map']
            return ''

        t = template.Template(simple_template_string)
        with mock.patch('easy_maps.templatetags.easy_maps_tags.render_to_string', get_map_context_instance):
            t.render(template.Context({}))

        self.assertEqual(address[0].latitude, AddressTests.fake_default_center[0])
        self.assertEqual(address[0].longitude, AddressTests.fake_default_center[1])

    @override_settings(EASY_MAPS_CENTER=fake_default_center)
    def test_normal_address(self):
        """If we pass an address don't use the defaults"""
        n_addresses_before = len(Address.objects.all())

        a = "Ekaterinburg, Mira 33"
        simple_template_string = """{%% load easy_maps_tags %%}
        {%% easy_map "%s" 500 500 10 %%}
        """ % a
        t = template.Template(simple_template_string)
        t.render(template.Context({}))

        address = Address.objects.get(address=a)
        self.assertNotEqual(address.latitude, AddressTests.fake_default_center[0])
        self.assertNotEqual(address.longitude, AddressTests.fake_default_center[1])

        n_addresses_after = len(Address.objects.all())

        self.assertEqual(n_addresses_after, n_addresses_before + 1)

    @override_settings(EASY_MAPS_CENTER=fake_default_center)
    def test_use_address_instance(self):
        """It's possible to pass directly to the easy_map tag an Address instance.

        This test checks also that the database is not hit.
        """
        # create a fake address
        Address.objects.create(address='fake')
        n_addresses_before = len(Address.objects.all())

        simple_template_string = """{% load easy_maps_tags %}
        {% easy_map address 500 500 10 %}
        """
        address = Address.objects.all()[0]
        t = template.Template(simple_template_string)
        # this assert works only from django1.3
        ctx = template.Context({'address': address,})
        self.assertNumQueries(0, lambda: t.render(ctx))

        n_addresses_after = len(Address.objects.all())

        # no Address is created in the process
        self.assertEqual(n_addresses_after, n_addresses_before)



from __future__ import absolute_import
import logging

from django.conf import settings
from django.db import models

from . import geocode

logger = logging.getLogger(__name__)


class Address(models.Model):
    address = models.CharField(max_length=255, db_index=True)
    computed_address = models.CharField(max_length=255, null=True, blank=True)
    latitude = models.FloatField(null=True, blank=True)
    longitude = models.FloatField(null=True, blank=True)
    geocode_error = models.BooleanField(default=False)

    def fill_geocode_data(self):
        if not self.address:
            self.geocode_error = True
            return
        try:
            do_geocode = getattr(settings, "EASY_MAPS_GEOCODE", geocode.google_v3)
            self.computed_address, (self.latitude, self.longitude,) = do_geocode(self.address)
            self.geocode_error = False
        except geocode.Error as e:
            try:
                logger.error(e)
            except Exception:
                logger.error("Geocoding error for address %s", self.address)

            self.geocode_error = True
            # TODO: store the exception

    def save(self, *args, **kwargs):
        # fill geocode data if it is unknown
        if (self.longitude is None) or (self.latitude is None):
            self.fill_geocode_data()
        super(Address, self).save(*args, **kwargs)

    def __unicode__(self):
        return self.address

    class Meta:
        verbose_name = "EasyMaps Address"
        verbose_name_plural = "Address Geocoding Cache"




# -*- coding: utf-8 -*-
from __future__ import absolute_import
from django.utils.encoding import smart_str
from geopy import geocoders
from geopy.geocoders.base import GeocoderResultError

class Error(Exception):
    pass


def google_v3(address):
    """
    Given an address, return ``(computed_address, (latitude, longitude))``
    tuple using Google Geocoding API v3.
    """
    try:
        g = geocoders.GoogleV3()
        address = smart_str(address)
        return g.geocode(address, exactly_one=False)[0]
    except (UnboundLocalError, ValueError, GeocoderResultError) as e:
        raise Error(e)



import os, sys
PROJECT_ROOT = os.path.dirname(os.path.abspath(__file__))
sys.path.insert(0, os.path.abspath(os.path.join(PROJECT_ROOT, '..')))

DEBUG = True
DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': 'db.sqlite',
    }
}
ROOT_URLCONF = 'urls'
TEMPLATE_DIRS = (os.path.join(PROJECT_ROOT, 'templates'),)


STATIC_URL = '/static/'
STATIC_ROOT = os.path.abspath(os.path.join(PROJECT_ROOT, 'static'))

SECRET_KEY = 'khkhgfkuydf-t734-435u3sdfgksadg-fo[7trHGchgsdf'

INSTALLED_APPS=(
    'django.contrib.contenttypes',
    'django.contrib.messages',
    'django.contrib.sessions',
    'django.contrib.auth',
    'django.contrib.admin',
    'django.contrib.staticfiles',
    'easy_maps',
    'test_app',
#    'devserver',
    'south',
)

EASY_MAPS_CENTER = (45.0677201, 7.6825531)



from django.contrib import admin
from django import forms

from .widgets import AddressWithMapWidget

class AddressAdmin(admin.ModelAdmin):
    list_display = ['address', 'computed_address', 'latitude', 'longitude', 'geocode_error']
    list_filter = ['geocode_error']
    search_fields = ['address']

    class form(forms.ModelForm):
        class Meta:
            widgets = {
                'address': AddressWithMapWidget({'class': 'vTextField'})
            }



from __future__ import absolute_import
from django.forms import TextInput
from django import template

class AddressWithMapWidget(TextInput):
    width = 700
    height = 200
    zoom = 16

    def render(self, name, value, attrs=None):
        tpl = "{{% load easy_maps_tags %}}" \
              "{{% easy_map address {0.width} {0.height} {0.zoom} %}}".format(self)
        map_template = template.Template(tpl)
        context = template.Context({'address': value})

        default_html = super(AddressWithMapWidget, self).render(name, value, attrs)
        return default_html + map_template.render(context)



# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        
        # Deleting field 'Address.longtitude'
        db.delete_column('easy_maps_address', 'longtitude')

    def backwards(self, orm):
        
        # Adding field 'Address.longtitude'
        db.add_column('easy_maps_address', 'longtitude', self.gf('django.db.models.fields.FloatField')(null=True, blank=True), keep_default=False)


    models = {
        'easy_maps.address': {
            'Meta': {'object_name': 'Address'},
            'address': ('django.db.models.fields.CharField', [], {'max_length': '255', 'db_index': 'True'}),
            'computed_address': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'geocode_error': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'latitude': ('django.db.models.fields.FloatField', [], {'null': 'True', 'blank': 'True'}),
            'longitude': ('django.db.models.fields.FloatField', [], {'null': 'True', 'blank': 'True'})
        }
    }

    complete_apps = ['easy_maps']



from django.conf.urls import patterns, url, include
from django.contrib import admin
from django.shortcuts import render_to_response


admin.autodiscover()

def index(request):
    return render_to_response('index.html')


urlpatterns = patterns('',
    url(r'^$', index, name='index'),
    url(r'^admin/', include(admin.site.urls)),
)



#coding: utf-8
from django import template
from django.template.loader import render_to_string
from easy_maps.models import Address
from django.conf import settings

register = template.Library()

@register.tag
def easy_map(parser, token):
    """
    The syntax:
        {% easy_map <address> [<width> <height>] [<zoom>] [using <template_name>] %}

    The "address" parameter can be an Address instance or a string describing it.
    If an address is not found a new entry is created in the database.
    """
    width, height, zoom, template_name = None, None, None, None
    params = token.split_contents()

    # pop the template name
    if params[-2] == 'using':
        template_name = params[-1]
        params = params[:-2]

    if len(params) < 2:
        raise template.TemplateSyntaxError('easy_map tag requires address argument')

    address = params[1]

    if len(params) == 4:
        width, height = params[2], params[3]
    elif len(params) == 5:
        width, height, zoom = params[2], params[3], params[4]
    elif len(params) == 3 or len(params) > 5:
        raise template.TemplateSyntaxError('easy_map tag has the following syntax: '
                   '{% easy_map <address> <width> <height> [zoom] [using <template_name>] %}')
    return EasyMapNode(address, width, height, zoom, template_name)


class EasyMapNode(template.Node):
    def __init__(self, address, width, height, zoom, template_name):
        self.address = template.Variable(address)
        self.width = width or ''
        self.height = height or ''
        self.zoom = zoom or 16
        self.template_name = template.Variable(template_name or '"easy_maps/map.html"')

    def get_map(self, address):
        if isinstance(address, Address):
            return address

        if address == '':
            map_ = Address(latitude=settings.EASY_MAPS_CENTER[0],
                           longitude=settings.EASY_MAPS_CENTER[1])
        else:
            map_, _ = Address.objects.get_or_create(address=address)

        return map_


    def render(self, context):
        try:
            address = self.address.resolve(context)
            template_name = self.template_name.resolve(context)
            map_ = self.get_map(address)

            context.update({
                'map': map_,
                'width': self.width,
                'height': self.height,
                'zoom': self.zoom,
                'template_name': template_name
            })
            return render_to_string(template_name, context_instance=context)
        except template.VariableDoesNotExist:
            return ''



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from django.contrib import admin
from easy_maps.admin import AddressAdmin
from easy_maps.models import Address

admin.site.register(Address, AddressAdmin)



# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        
        # Adding model 'Address'
        db.create_table('easy_maps_address', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('address', self.gf('django.db.models.fields.CharField')(max_length=255, db_index=True)),
            ('computed_address', self.gf('django.db.models.fields.CharField')(max_length=255, null=True, blank=True)),
            ('latitude', self.gf('django.db.models.fields.FloatField')(null=True, blank=True)),
            ('longtitude', self.gf('django.db.models.fields.FloatField')(null=True, blank=True)),
            ('geocode_error', self.gf('django.db.models.fields.BooleanField')(default=False)),
        ))
        db.send_create_signal('easy_maps', ['Address'])


    def backwards(self, orm):
        
        # Deleting model 'Address'
        db.delete_table('easy_maps_address')


    models = {
        'easy_maps.address': {
            'Meta': {'object_name': 'Address'},
            'address': ('django.db.models.fields.CharField', [], {'max_length': '255', 'db_index': 'True'}),
            'computed_address': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'geocode_error': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'latitude': ('django.db.models.fields.FloatField', [], {'null': 'True', 'blank': 'True'}),
            'longtitude': ('django.db.models.fields.FloatField', [], {'null': 'True', 'blank': 'True'})
        }
    }

    complete_apps = ['easy_maps']
